# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from airflow import DAG
from airflow.contrib.operators import gcs_to_bq, kubernetes_pod_operator

default_args = {
    "owner": "Google",
    "depends_on_past": False,
    "start_date": "2021-03-01",
}


with DAG(
    dag_id="irs_990.irs_990_pf_2015",
    default_args=default_args,
    max_active_runs=1,
    schedule_interval="@daily",
    catchup=False,
    default_view="graph",
) as dag:

    # Run CSV transform within kubernetes pod
    irs_990_pf_2015_transform_csv = kubernetes_pod_operator.KubernetesPodOperator(
        task_id="irs_990_pf_2015_transform_csv",
        startup_timeout_seconds=600,
        name="irs_990_pf_2015",
        affinity={
            "nodeAffinity": {
                "requiredDuringSchedulingIgnoredDuringExecution": {
                    "nodeSelectorTerms": [
                        {
                            "matchExpressions": [
                                {
                                    "key": "cloud.google.com/gke-nodepool",
                                    "operator": "In",
                                    "values": ["pool-e2-standard-4"],
                                }
                            ]
                        }
                    ]
                }
            }
        },
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.irs_990.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "https://www.irs.gov/pub/irs-soi/15eofinextract990pf.dat",
            "SOURCE_FILE": "files/data.dat",
            "TARGET_FILE": "files/data_output.csv",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "data/irs_990/irs_990_pf_2015/data_output.csv",
            "PIPELINE_NAME": "irs_990_pf_2015",
            "CSV_HEADERS": '["ein","elf","tax_prd","eostatus","tax_yr","operatingcd","subcd","fairmrktvalamt","grscontrgifts","schedbind","intrstrvnue","dividndsamt","grsrents","grsslspramt","costsold","grsprofitbus","otherincamt","totrcptperbks","compofficers","pensplemplbenf","legalfeesamt","accountingfees","interestamt","depreciationamt","occupancyamt","travlconfmtngs","printingpubl","topradmnexpnsa","contrpdpbks","totexpnspbks","excessrcpts","totrcptnetinc","topradmnexpnsb","totexpnsnetinc","netinvstinc","trcptadjnetinc","totexpnsadjnet","adjnetinc","topradmnexpnsd","totexpnsexempt","othrcashamt","invstgovtoblig","invstcorpstk","invstcorpbnd","totinvstsec","mrtgloans","othrinvstend","othrassetseoy","totassetsend","mrtgnotespay","othrliabltseoy","totliabend","tfundnworth","fairmrktvaleoy","totexcapgnls","totexcapgn","totexcapls","invstexcisetx","sec4940notxcd","sec4940redtxcd","sect511tx","subtitleatx","totaxpyr","esttaxcr","txwithldsrc","txpaidf2758","erronbkupwthld","estpnlty","taxdue","overpay","crelamt","infleg","actnotpr","chgnprvrptcd","filedf990tcd","contractncd","furnishcpycd","claimstatcd","cntrbtrstxyrcd","distribdafcd","orgcmplypubcd","filedlf1041ind","propexchcd","brwlndmnycd","furngoodscd","paidcmpncd","transfercd","agremkpaycd","exceptactsind","prioractvcd","undistrinccd","applyprovind","dirindirintcd","excesshldcd","invstjexmptcd","prevjexmptcd","propgndacd","ipubelectcd","grntindivcd","nchrtygrntcd","nreligiouscd","excptransind","rfprsnlbnftind","pyprsnlbnftind","tfairmrktunuse","valncharitassets","cmpmininvstret","distribamt","undistribincyr","adjnetinccola","adjnetinccolb","adjnetinccolc","adjnetinccold","adjnetinctot","qlfydistriba","qlfydistribb","qlfydistribc","qlfydistribd","qlfydistribtot","valassetscola","valassetscolb","valassetscolc","valassetscold","valassetstot","qlfyasseta","qlfyassetb","qlfyassetc","qlfyassetd","qlfyassettot","endwmntscola","endwmntscolb","endwmntscolc","endwmntscold","endwmntstot","totsuprtcola","totsuprtcolb","totsuprtcolc","totsuprtcold","totsuprttot","pubsuprtcola","pubsuprtcolb","pubsuprtcolc","pubsuprtcold","pubsuprttot","grsinvstinca","grsinvstincb","grsinvstincc","grsinvstincd","grsinvstinctot","grntapprvfut","progsrvcacold","progsrvcacole","progsrvcbcold","progsrvcbcole","progsrvcccold","progsrvcccole","progsrvcdcold","progsrvcdcole","progsrvcecold","progsrvcecole","progsrvcfcold","progsrvcfcole","progsrvcgcold","progsrvcgcole","membershpduesd","membershpduese","intonsvngsd","intonsvngse","dvdndsintd","dvdndsinte","trnsfrcashcd","trnsothasstscd","salesasstscd","prchsasstscd","rentlsfacltscd","reimbrsmntscd","loansguarcd","perfservicescd","sharngasstscd"]',
            "RENAME_MAPPINGS": '{"ELF": "elf","ELFCD": "elf","EIN": "ein","TAX_PRD": "tax_prd","EOSTATUS": "eostatus","TAX_YR": "tax_yr","OPERATINGCD": "operatingcd","SUBCD": "subcd","FAIRMRKTVALAMT": "fairmrktvalamt","GRSCONTRGIFTS": "grscontrgifts","SCHEDBIND": "schedbind","INTRSTRVNUE": "intrstrvnue","DIVIDNDSAMT": "dividndsamt","GRSRENTS": "grsrents","GRSSLSPRAMT": "grsslspramt","COSTSOLD": "costsold","GRSPROFITBUS": "grsprofitbus","OTHERINCAMT": "otherincamt","TOTRCPTPERBKS": "totrcptperbks","COMPOFFICERS": "compofficers","PENSPLEMPLBENF": "pensplemplbenf","LEGALFEESAMT": "legalfeesamt","ACCOUNTINGFEES": "accountingfees","INTERESTAMT": "interestamt","DEPRECIATIONAMT": "depreciationamt","OCCUPANCYAMT": "occupancyamt","TRAVLCONFMTNGS": "travlconfmtngs","PRINTINGPUBL": "printingpubl","TOPRADMNEXPNSA": "topradmnexpnsa","CONTRPDPBKS": "contrpdpbks","TOTEXPNSPBKS": "totexpnspbks","EXCESSRCPTS": "excessrcpts","TOTRCPTNETINC": "totrcptnetinc","TOPRADMNEXPNSB": "topradmnexpnsb","TOTEXPNSNETINC": "totexpnsnetinc","NETINVSTINC": "netinvstinc","TRCPTADJNETINC": "trcptadjnetinc","TOTEXPNSADJNET": "totexpnsadjnet","ADJNETINC": "adjnetinc","TOPRADMNEXPNSD": "topradmnexpnsd","TOTEXPNSEXEMPT": "totexpnsexempt","OTHRCASHAMT": "othrcashamt","INVSTGOVTOBLIG": "invstgovtoblig","INVSTCORPSTK": "invstcorpstk","INVSTCORPBND": "invstcorpbnd","TOTINVSTSEC": "totinvstsec","MRTGLOANS": "mrtgloans","OTHRINVSTEND": "othrinvstend","OTHRASSETSEOY": "othrassetseoy","TOTASSETSEND": "totassetsend","MRTGNOTESPAY": "mrtgnotespay","OTHRLIABLTSEOY": "othrliabltseoy","TOTLIABEND": "totliabend","TFUNDNWORTH": "tfundnworth","FAIRMRKTVALEOY": "fairmrktvaleoy","TOTEXCAPGNLS": "totexcapgnls","TOTEXCAPGN": "totexcapgn","TOTEXCAPLS": "totexcapls","INVSTEXCISETX": "invstexcisetx","SEC4940NOTXCD": "sec4940notxcd","SEC4940REDTXCD": "sec4940redtxcd","SECT511TX": "sect511tx","SUBTITLEATX": "subtitleatx","TOTAXPYR": "totaxpyr","ESTTAXCR": "esttaxcr","TXWITHLDSRC": "txwithldsrc","TXPAIDF2758": "txpaidf2758","ERRONBKUPWTHLD": "erronbkupwthld","ESTPNLTY": "estpnlty","TAXDUE": "taxdue","OVERPAY": "overpay","CRELAMT": "crelamt","INFLEG": "infleg","ACTNOTPR": "actnotpr","CHGNPRVRPTCD": "chgnprvrptcd","FILEDF990TCD": "filedf990tcd","CONTRACTNCD": "contractncd","FURNISHCPYCD": "furnishcpycd","CLAIMSTATCD": "claimstatcd","CNTRBTRSTXYRCD": "cntrbtrstxyrcd","DISTRIBDAFCD": "distribdafcd","ACQDRINDRINTCD": "distribdafcd","ORGCMPLYPUBCD": "orgcmplypubcd","FILEDLF1041IND": "filedlf1041ind","PROPEXCHCD": "propexchcd","BRWLNDMNYCD": "brwlndmnycd","FURNGOODSCD": "furngoodscd","PAIDCMPNCD": "paidcmpncd","TRANSFERCD": "transfercd","AGREMKPAYCD": "agremkpaycd","EXCEPTACTSIND": "exceptactsind","PRIORACTVCD": "prioractvcd","UNDISTRINCCD": "undistrinccd","APPLYPROVIND": "applyprovind","DIRINDIRINTCD": "dirindirintcd","EXCESSHLDCD": "excesshldcd","INVSTJEXMPTCD": "invstjexmptcd","PREVJEXMPTCD": "prevjexmptcd","PROPGNDACD": "propgndacd","IPUBELECTCD": "ipubelectcd","GRNTINDIVCD": "grntindivcd","NCHRTYGRNTCD": "nchrtygrntcd","NRELIGIOUSCD": "nreligiouscd","EXCPTRANSIND": "excptransind","RFPRSNLBNFTIND": "rfprsnlbnftind","PYPRSNLBNFTIND": "pyprsnlbnftind","TFAIRMRKTUNUSE": "tfairmrktunuse","VALNCHARITASSETS": "valncharitassets","CMPMININVSTRET": "cmpmininvstret","DISTRIBAMT": "distribamt","UNDISTRIBINCYR": "undistribincyr","ADJNETINCCOLA": "adjnetinccola","ADJNETINCCOLB": "adjnetinccolb","ADJNETINCCOLC": "adjnetinccolc","ADJNETINCCOLD": "adjnetinccold","ADJNETINCTOT": "adjnetinctot","QLFYDISTRIBA": "qlfydistriba","QLFYDISTRIBB": "qlfydistribb","QLFYDISTRIBC": "qlfydistribc","QLFYDISTRIBD": "qlfydistribd","QLFYDISTRIBTOT": "qlfydistribtot","VALASSETSCOLA": "valassetscola","VALASSETSCOLB": "valassetscolb","VALASSETSCOLC": "valassetscolc","VALASSETSCOLD": "valassetscold","VALASSETSTOT": "valassetstot","QLFYASSETA": "qlfyasseta","QLFYASSETB": "qlfyassetb","QLFYASSETC": "qlfyassetc","QLFYASSETD": "qlfyassetd","QLFYASSETTOT": "qlfyassettot","ENDWMNTSCOLA": "endwmntscola","ENDWMNTSCOLB": "endwmntscolb","ENDWMNTSCOLC": "endwmntscolc","ENDWMNTSCOLD": "endwmntscold","ENDWMNTSTOT": "endwmntstot","TOTSUPRTCOLA": "totsuprtcola","TOTSUPRTCOLB": "totsuprtcolb","TOTSUPRTCOLC": "totsuprtcolc","TOTSUPRTCOLD": "totsuprtcold","TOTSUPRTTOT": "totsuprttot","PUBSUPRTCOLA": "pubsuprtcola","PUBSUPRTCOLB": "pubsuprtcolb","PUBSUPRTCOLC": "pubsuprtcolc","PUBSUPRTCOLD": "pubsuprtcold","PUBSUPRTTOT": "pubsuprttot","GRSINVSTINCA": "grsinvstinca","GRSINVSTINCB": "grsinvstincb","GRSINVSTINCC": "grsinvstincc","GRSINVSTINCD": "grsinvstincd","GRSINVSTINCTOT": "grsinvstinctot","GRNTAPPRVFUT": "grntapprvfut","PROGSRVCACOLD": "progsrvcacold","PROGSRVCACOLE": "progsrvcacole","PROGSRVCBCOLD": "progsrvcbcold","PROGSRVCBCOLE": "progsrvcbcole","PROGSRVCCCOLD": "progsrvcccold","PROGSRVCCCOLE": "progsrvcccole","PROGSRVCDCOLD": "progsrvcdcold","PROGSRVCDCOLE": "progsrvcdcole","PROGSRVCECOLD": "progsrvcecold","PROGSRVCECOLE": "progsrvcecole","PROGSRVCFCOLD": "progsrvcfcold","PROGSRVCFCOLE": "progsrvcfcole","PROGSRVCGCOLD": "progsrvcgcold","PROGSRVCGCOLE": "progsrvcgcole","MEMBERSHPDUESD": "membershpduesd","MEMBERSHPDUESE": "membershpduese","INTONSVNGSD": "intonsvngsd","INTONSVNGSE": "intonsvngse","DVDNDSINTD": "dvdndsintd","DVDNDSINTE": "dvdndsinte","TRNSFRCASHCD": "trnsfrcashcd","TRNSOTHASSTSCD": "trnsothasstscd","SALESASSTSCD": "salesasstscd","PRCHSASSTSCD": "prchsasstscd","RENTLSFACLTSCD": "rentlsfacltscd","REIMBRSMNTSCD": "reimbrsmntscd","LOANSGUARCD": "loansguarcd","PERFSERVICESCD": "perfservicescd","SHARNGASSTSCD": "sharngasstscd"}',
        },
        resources={"request_memory": "2G", "request_cpu": "1"},
    )

    # Task to load CSV data to a BigQuery table
    load_irs_990_pf_2015_to_bq = gcs_to_bq.GoogleCloudStorageToBigQueryOperator(
        task_id="load_irs_990_pf_2015_to_bq",
        bucket="{{ var.value.composer_bucket }}",
        source_objects=["data/irs_990/irs_990_pf_2015/data_output.csv"],
        source_format="CSV",
        destination_project_dataset_table="irs_990.irs_990_pf_2015",
        skip_leading_rows=1,
        write_disposition="WRITE_TRUNCATE",
        schema_fields=[
            {
                "name": "ein",
                "type": "string",
                "description": "Employer Identification Number",
                "mode": "required",
            },
            {
                "name": "elf",
                "type": "string",
                "description": "E-file indicator",
                "mode": "nullable",
            },
            {
                "name": "tax_prd",
                "type": "string",
                "description": "Tax period (YYYYMM format)",
                "mode": "nullable",
            },
            {
                "name": "eostatus",
                "type": "string",
                "description": "EO Status Code",
                "mode": "nullable",
            },
            {
                "name": "tax_yr",
                "type": "integer",
                "description": "SOI Year",
                "mode": "nullable",
            },
            {
                "name": "operatingcd",
                "type": "string",
                "description": "Operating foundation code",
                "mode": "nullable",
            },
            {
                "name": "subcd",
                "type": "string",
                "description": "Subsection code",
                "mode": "nullable",
            },
            {
                "name": "fairmrktvalamt",
                "type": "integer",
                "description": "Total assets – e-o-y fair market valu",
                "mode": "nullable",
            },
            {
                "name": "grscontrgifts",
                "type": "integer",
                "description": "Contributions received",
                "mode": "nullable",
            },
            {
                "name": "schedbind",
                "type": "string",
                "description": "Schedule B indicator",
                "mode": "nullable",
            },
            {
                "name": "intrstrvnue",
                "type": "integer",
                "description": "Interest revenue",
                "mode": "nullable",
            },
            {
                "name": "dividndsamt",
                "type": "integer",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "grsrents",
                "type": "integer",
                "description": "Gross rents",
                "mode": "nullable",
            },
            {
                "name": "grsslspramt",
                "type": "integer",
                "description": "Gross sales price for assets",
                "mode": "nullable",
            },
            {
                "name": "costsold",
                "type": "integer",
                "description": "Cost-of-goods-sold",
                "mode": "nullable",
            },
            {
                "name": "grsprofitbus",
                "type": "integer",
                "description": "Gross profit",
                "mode": "nullable",
            },
            {
                "name": "otherincamt",
                "type": "integer",
                "description": "Other income",
                "mode": "nullable",
            },
            {
                "name": "totrcptperbks",
                "type": "integer",
                "description": "Total revenue",
                "mode": "nullable",
            },
            {
                "name": "compofficers",
                "type": "integer",
                "description": "Compensation of officers",
                "mode": "nullable",
            },
            {
                "name": "pensplemplbenf",
                "type": "integer",
                "description": "Pension plans employee benefits",
                "mode": "nullable",
            },
            {
                "name": "legalfeesamt",
                "type": "integer",
                "description": "Legal fees",
                "mode": "nullable",
            },
            {
                "name": "accountingfees",
                "type": "integer",
                "description": "Accounting fees",
                "mode": "nullable",
            },
            {
                "name": "interestamt",
                "type": "integer",
                "description": "Interest",
                "mode": "nullable",
            },
            {
                "name": "depreciationamt",
                "type": "integer",
                "description": "Depreciation and depletion",
                "mode": "nullable",
            },
            {
                "name": "occupancyamt",
                "type": "integer",
                "description": "Occupancy",
                "mode": "nullable",
            },
            {
                "name": "travlconfmtngs",
                "type": "integer",
                "description": "Travel conferences and meetings",
                "mode": "nullable",
            },
            {
                "name": "printingpubl",
                "type": "integer",
                "description": "Printing and publications",
                "mode": "nullable",
            },
            {
                "name": "topradmnexpnsa",
                "type": "integer",
                "description": "Total operating and administrative expenses column a",
                "mode": "nullable",
            },
            {
                "name": "contrpdpbks",
                "type": "integer",
                "description": "Contributions gifts grants paid",
                "mode": "nullable",
            },
            {
                "name": "totexpnspbks",
                "type": "integer",
                "description": "Total expenses",
                "mode": "nullable",
            },
            {
                "name": "excessrcpts",
                "type": "integer",
                "description": "Net income less deficit",
                "mode": "nullable",
            },
            {
                "name": "totrcptnetinc",
                "type": "integer",
                "description": "Total receipts net investment income",
                "mode": "nullable",
            },
            {
                "name": "topradmnexpnsb",
                "type": "integer",
                "description": "Total operating and administrative expenses column b",
                "mode": "nullable",
            },
            {
                "name": "totexpnsnetinc",
                "type": "integer",
                "description": "Total expenses net investment income",
                "mode": "nullable",
            },
            {
                "name": "netinvstinc",
                "type": "integer",
                "description": "Net investment income",
                "mode": "nullable",
            },
            {
                "name": "trcptadjnetinc",
                "type": "integer",
                "description": "Total receipts adjusted net income",
                "mode": "nullable",
            },
            {
                "name": "totexpnsadjnet",
                "type": "integer",
                "description": "Total expenses adjusted net income",
                "mode": "nullable",
            },
            {
                "name": "adjnetinc",
                "type": "integer",
                "description": "Adjusted net income",
                "mode": "nullable",
            },
            {
                "name": "topradmnexpnsd",
                "type": "integer",
                "description": "Total operating and administrative expenses column d",
                "mode": "nullable",
            },
            {
                "name": "totexpnsexempt",
                "type": "integer",
                "description": "Total expenses – exempt purpose",
                "mode": "nullable",
            },
            {
                "name": "othrcashamt",
                "type": "integer",
                "description": "Cash non-interest-bearing – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "invstgovtoblig",
                "type": "integer",
                "description": "Investments in U.S. & state government obligations – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "invstcorpstk",
                "type": "integer",
                "description": "Investments in corporate stock – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "invstcorpbnd",
                "type": "integer",
                "description": "Investments in corporate bonds– e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "totinvstsec",
                "type": "integer",
                "description": "Total investments in securities – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "mrtgloans",
                "type": "integer",
                "description": "Investments mortgage loans – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "othrinvstend",
                "type": "integer",
                "description": "Other investments – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "othrassetseoy",
                "type": "integer",
                "description": "Other assets – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "totassetsend",
                "type": "integer",
                "description": "Total assets – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "mrtgnotespay",
                "type": "integer",
                "description": "Mortgage loans payable – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "othrliabltseoy",
                "type": "integer",
                "description": "Other liabilities – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "totliabend",
                "type": "integer",
                "description": "Total liabilities – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "tfundnworth",
                "type": "integer",
                "description": "Total fund net worth – e-o-y book value",
                "mode": "nullable",
            },
            {
                "name": "fairmrktvaleoy",
                "type": "integer",
                "description": "Total assets – e-o-y fair market value",
                "mode": "nullable",
            },
            {
                "name": "totexcapgnls",
                "type": "integer",
                "description": "Capital gain net income",
                "mode": "nullable",
            },
            {
                "name": "totexcapgn",
                "type": "integer",
                "description": "Net gain – sales of assets",
                "mode": "nullable",
            },
            {
                "name": "totexcapls",
                "type": "integer",
                "description": "Net loss – sales of assets",
                "mode": "nullable",
            },
            {
                "name": "invstexcisetx",
                "type": "integer",
                "description": "Excise tax on net investment income",
                "mode": "nullable",
            },
            {
                "name": "sec4940notxcd",
                "type": "string",
                "description": "Section 4940 – no tax",
                "mode": "nullable",
            },
            {
                "name": "sec4940redtxcd",
                "type": "string",
                "description": "Section 4940 – 1 % tax",
                "mode": "nullable",
            },
            {
                "name": "sect511tx",
                "type": "integer",
                "description": "Section 511 tax",
                "mode": "nullable",
            },
            {
                "name": "subtitleatx",
                "type": "integer",
                "description": "Subtitle A tax",
                "mode": "nullable",
            },
            {
                "name": "totaxpyr",
                "type": "integer",
                "description": "Total excise tax",
                "mode": "nullable",
            },
            {
                "name": "esttaxcr",
                "type": "integer",
                "description": "Estimated tax credit",
                "mode": "nullable",
            },
            {
                "name": "txwithldsrc",
                "type": "integer",
                "description": "Tax withheld at source",
                "mode": "nullable",
            },
            {
                "name": "txpaidf2758",
                "type": "integer",
                "description": "Tax paid with Form 2758 (filing extension)",
                "mode": "nullable",
            },
            {
                "name": "erronbkupwthld",
                "type": "integer",
                "description": "Erroneous backup withholding credit amount",
                "mode": "nullable",
            },
            {
                "name": "estpnlty",
                "type": "integer",
                "description": "Estimated tax penalty",
                "mode": "nullable",
            },
            {
                "name": "taxdue",
                "type": "integer",
                "description": "Tax due",
                "mode": "nullable",
            },
            {
                "name": "overpay",
                "type": "integer",
                "description": "Overpayment",
                "mode": "nullable",
            },
            {
                "name": "crelamt",
                "type": "integer",
                "description": "Credit elect amount",
                "mode": "nullable",
            },
            {
                "name": "infleg",
                "type": "string",
                "description": "Influence legislation?",
                "mode": "nullable",
            },
            {
                "name": "actnotpr",
                "type": "string",
                "description": "Activities not previously reported?",
                "mode": "nullable",
            },
            {
                "name": "chgnprvrptcd",
                "type": "string",
                "description": "Changes not previously reported?",
                "mode": "nullable",
            },
            {
                "name": "filedf990tcd",
                "type": "string",
                "description": "Filed 990-T?",
                "mode": "nullable",
            },
            {
                "name": "contractncd",
                "type": "string",
                "description": "Contraction?",
                "mode": "nullable",
            },
            {
                "name": "furnishcpycd",
                "type": "string",
                "description": "Furnished copy to Attorney General?",
                "mode": "nullable",
            },
            {
                "name": "claimstatcd",
                "type": "string",
                "description": "Claiming status?",
                "mode": "nullable",
            },
            {
                "name": "cntrbtrstxyrcd",
                "type": "string",
                "description": "Substantial contributors?",
                "mode": "nullable",
            },
            {
                "name": "distribdafcd",
                "type": "string",
                "description": "Distribution to donor advised fund with advisory privileges?",
                "mode": "nullable",
            },
            {
                "name": "orgcmplypubcd",
                "type": "string",
                "description": "Comply with public inspection?",
                "mode": "nullable",
            },
            {
                "name": "filedlf1041ind",
                "type": "string",
                "description": "Comply with public inspection?",
                "mode": "nullable",
            },
            {
                "name": "propexchcd",
                "type": "string",
                "description": "Property exchange?",
                "mode": "nullable",
            },
            {
                "name": "brwlndmnycd",
                "type": "string",
                "description": "Borrow lend money?",
                "mode": "nullable",
            },
            {
                "name": "furngoodscd",
                "type": "string",
                "description": "Furnished goods?",
                "mode": "nullable",
            },
            {
                "name": "paidcmpncd",
                "type": "string",
                "description": "Paid compensation?",
                "mode": "nullable",
            },
            {
                "name": "transfercd",
                "type": "string",
                "description": "Transfer?",
                "mode": "nullable",
            },
            {
                "name": "agremkpaycd",
                "type": "string",
                "description": "Agree to make pay?",
                "mode": "nullable",
            },
            {
                "name": "exceptactsind",
                "type": "string",
                "description": "Acts fail to qualify under section 53.4941(d)-3?",
                "mode": "nullable",
            },
            {
                "name": "prioractvcd",
                "type": "string",
                "description": "Engage in acts in prior year?",
                "mode": "nullable",
            },
            {
                "name": "undistrinccd",
                "type": "string",
                "description": "Undistributed income?",
                "mode": "nullable",
            },
            {
                "name": "applyprovind",
                "type": "string",
                "description": "Not applying section 4942(a)(2) provisions?",
                "mode": "nullable",
            },
            {
                "name": "dirindirintcd",
                "type": "string",
                "description": "Direct indirect interest?",
                "mode": "nullable",
            },
            {
                "name": "excesshldcd",
                "type": "string",
                "description": "Excess business holdings?",
                "mode": "nullable",
            },
            {
                "name": "invstjexmptcd",
                "type": "string",
                "description": "Jeopardizing investments?",
                "mode": "nullable",
            },
            {
                "name": "prevjexmptcd",
                "type": "string",
                "description": "Prior year jeopardizing investments?",
                "mode": "nullable",
            },
            {
                "name": "propgndacd",
                "type": "string",
                "description": "Propaganda?",
                "mode": "nullable",
            },
            {
                "name": "ipubelectcd",
                "type": "string",
                "description": "Influence public election?",
                "mode": "nullable",
            },
            {
                "name": "grntindivcd",
                "type": "string",
                "description": "Grant individual?",
                "mode": "nullable",
            },
            {
                "name": "nchrtygrntcd",
                "type": "string",
                "description": "Non-charity grant?",
                "mode": "nullable",
            },
            {
                "name": "nreligiouscd",
                "type": "string",
                "description": "Non-religious?",
                "mode": "nullable",
            },
            {
                "name": "excptransind",
                "type": "string",
                "description": "Transactions fail to qualify under section 53.4945?",
                "mode": "nullable",
            },
            {
                "name": "rfprsnlbnftind",
                "type": "string",
                "description": "Receive funds to pay premiums on personal benefit contract?",
                "mode": "nullable",
            },
            {
                "name": "pyprsnlbnftind",
                "type": "string",
                "description": "Pay premiums on personal benefit contract?",
                "mode": "nullable",
            },
            {
                "name": "tfairmrktunuse",
                "type": "integer",
                "description": "Fair market value of assets not used for charitable purposes",
                "mode": "nullable",
            },
            {
                "name": "valncharitassets",
                "type": "integer",
                "description": "Net value of noncharitable-use assets",
                "mode": "nullable",
            },
            {
                "name": "cmpmininvstret",
                "type": "integer",
                "description": "Minimum investment return",
                "mode": "nullable",
            },
            {
                "name": "distribamt",
                "type": "integer",
                "description": "Distributable amount",
                "mode": "nullable",
            },
            {
                "name": "undistribincyr",
                "type": "integer",
                "description": "Undistributed income",
                "mode": "nullable",
            },
            {
                "name": "adjnetinccola",
                "type": "integer",
                "description": "Adjusted net income column a",
                "mode": "nullable",
            },
            {
                "name": "adjnetinccolb",
                "type": "integer",
                "description": "Adjusted net income column b",
                "mode": "nullable",
            },
            {
                "name": "adjnetinccolc",
                "type": "integer",
                "description": "Adjusted net income column c",
                "mode": "nullable",
            },
            {
                "name": "adjnetinccold",
                "type": "integer",
                "description": "Adjusted net income column d",
                "mode": "nullable",
            },
            {
                "name": "adjnetinctot",
                "type": "integer",
                "description": "Adjusted net income total",
                "mode": "nullable",
            },
            {
                "name": "qlfydistriba",
                "type": "integer",
                "description": "Qualifying distributions column a",
                "mode": "nullable",
            },
            {
                "name": "qlfydistribb",
                "type": "integer",
                "description": "Qualifying distributions column b",
                "mode": "nullable",
            },
            {
                "name": "qlfydistribc",
                "type": "integer",
                "description": "Qualifying distributions column c",
                "mode": "nullable",
            },
            {
                "name": "qlfydistribd",
                "type": "integer",
                "description": "Qualifying distributions column d",
                "mode": "nullable",
            },
            {
                "name": "qlfydistribtot",
                "type": "integer",
                "description": "Qualifying distributions total",
                "mode": "nullable",
            },
            {
                "name": "valassetscola",
                "type": "integer",
                "description": "Value assets column a",
                "mode": "nullable",
            },
            {
                "name": "valassetscolb",
                "type": "integer",
                "description": "Value assets column b",
                "mode": "nullable",
            },
            {
                "name": "valassetscolc",
                "type": "integer",
                "description": "Value assets column c",
                "mode": "nullable",
            },
            {
                "name": "valassetscold",
                "type": "integer",
                "description": "Value assets column d",
                "mode": "nullable",
            },
            {
                "name": "valassetstot",
                "type": "integer",
                "description": "Value assets total",
                "mode": "nullable",
            },
            {
                "name": "qlfyasseta",
                "type": "integer",
                "description": "Qualifying assets column a",
                "mode": "nullable",
            },
            {
                "name": "qlfyassetb",
                "type": "integer",
                "description": "Qualifying assets column b",
                "mode": "nullable",
            },
            {
                "name": "qlfyassetc",
                "type": "integer",
                "description": "Qualifying assets column c",
                "mode": "nullable",
            },
            {
                "name": "qlfyassetd",
                "type": "integer",
                "description": "Qualifying assets column d",
                "mode": "nullable",
            },
            {
                "name": "qlfyassettot",
                "type": "integer",
                "description": "Qualifying assets total",
                "mode": "nullable",
            },
            {
                "name": "endwmntscola",
                "type": "integer",
                "description": "Endowments column a",
                "mode": "nullable",
            },
            {
                "name": "endwmntscolb",
                "type": "integer",
                "description": "Endowments column b",
                "mode": "nullable",
            },
            {
                "name": "endwmntscolc",
                "type": "integer",
                "description": "Endowments column c",
                "mode": "nullable",
            },
            {
                "name": "endwmntscold",
                "type": "integer",
                "description": "Endowments column d",
                "mode": "nullable",
            },
            {
                "name": "endwmntstot",
                "type": "integer",
                "description": "Endowments total",
                "mode": "nullable",
            },
            {
                "name": "totsuprtcola",
                "type": "integer",
                "description": "Total support column a",
                "mode": "nullable",
            },
            {
                "name": "totsuprtcolb",
                "type": "integer",
                "description": "Total support column b",
                "mode": "nullable",
            },
            {
                "name": "totsuprtcolc",
                "type": "integer",
                "description": "Total support column c",
                "mode": "nullable",
            },
            {
                "name": "totsuprtcold",
                "type": "integer",
                "description": "Total support column d",
                "mode": "nullable",
            },
            {
                "name": "totsuprttot",
                "type": "integer",
                "description": "Total support total",
                "mode": "nullable",
            },
            {
                "name": "pubsuprtcola",
                "type": "integer",
                "description": "Public support column a",
                "mode": "nullable",
            },
            {
                "name": "pubsuprtcolb",
                "type": "integer",
                "description": "Public support column b",
                "mode": "nullable",
            },
            {
                "name": "pubsuprtcolc",
                "type": "integer",
                "description": "Public support column c",
                "mode": "nullable",
            },
            {
                "name": "pubsuprtcold",
                "type": "integer",
                "description": "Public support column d",
                "mode": "nullable",
            },
            {
                "name": "pubsuprttot",
                "type": "integer",
                "description": "Public support total",
                "mode": "nullable",
            },
            {
                "name": "grsinvstinca",
                "type": "integer",
                "description": "Gross investment income column a",
                "mode": "nullable",
            },
            {
                "name": "grsinvstincb",
                "type": "integer",
                "description": "Gross investment income column b",
                "mode": "nullable",
            },
            {
                "name": "grsinvstincc",
                "type": "integer",
                "description": "Gross investment income column c",
                "mode": "nullable",
            },
            {
                "name": "grsinvstincd",
                "type": "integer",
                "description": "Gross investment income column d",
                "mode": "nullable",
            },
            {
                "name": "grsinvstinctot",
                "type": "integer",
                "description": "Gross investment income total",
                "mode": "nullable",
            },
            {
                "name": "grntapprvfut",
                "type": "integer",
                "description": "Grants approved for future payment",
                "mode": "nullable",
            },
            {
                "name": "progsrvcacold",
                "type": "integer",
                "description": "Program service revenue line 1a (excluded)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcacole",
                "type": "integer",
                "description": "Program service revenue line 1a (exempt)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcbcold",
                "type": "integer",
                "description": "Program service revenue line 1b (excluded)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcbcole",
                "type": "integer",
                "description": "Program service revenue line 1b (exempt)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcccold",
                "type": "integer",
                "description": "Program service revenue line 1c (excluded)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcccole",
                "type": "integer",
                "description": "Program service revenue line 1c (exempt)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcdcold",
                "type": "integer",
                "description": "Program service revenue line 1d (excluded)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcdcole",
                "type": "integer",
                "description": "Program service revenue line 1d (exempt)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcecold",
                "type": "integer",
                "description": "Program service revenue line 1e (excluded)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcecole",
                "type": "integer",
                "description": "Program service revenue line 1e (exempt)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcfcold",
                "type": "integer",
                "description": "Program service revenue line 1f (excluded)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcfcole",
                "type": "integer",
                "description": "Program service revenue line 1f (exempt)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcgcold",
                "type": "integer",
                "description": "Program service revenue--fees and contracts from government line 1g (excluded)",
                "mode": "nullable",
            },
            {
                "name": "progsrvcgcole",
                "type": "integer",
                "description": "Program service revenue--fees and contracts from government line 1g (exempt)",
                "mode": "nullable",
            },
            {
                "name": "membershpduesd",
                "type": "integer",
                "description": "Membership dues and assessments (excluded)",
                "mode": "nullable",
            },
            {
                "name": "membershpduese",
                "type": "integer",
                "description": "Membership dues and assessments (exempt)",
                "mode": "nullable",
            },
            {
                "name": "intonsvngsd",
                "type": "integer",
                "description": "Interest on savings and temporary cash investments (excluded)",
                "mode": "nullable",
            },
            {
                "name": "intonsvngse",
                "type": "integer",
                "description": "Interest on savings and temporary cash investments (exempt)",
                "mode": "nullable",
            },
            {
                "name": "dvdndsintd",
                "type": "integer",
                "description": "Dividends and interest from securities (excluded)",
                "mode": "nullable",
            },
            {
                "name": "dvdndsinte",
                "type": "integer",
                "description": "Dividends and interest from securities (exempt)",
                "mode": "nullable",
            },
            {
                "name": "trnsfrcashcd",
                "type": "string",
                "description": "Transfer cash to noncharitable exempt organization?",
                "mode": "nullable",
            },
            {
                "name": "trnsothasstscd",
                "type": "string",
                "description": "Transfer other assets to noncharitable exempt organization?",
                "mode": "nullable",
            },
            {
                "name": "salesasstscd",
                "type": "string",
                "description": "Sale of assets to noncharitable exempt organization?",
                "mode": "nullable",
            },
            {
                "name": "prchsasstscd",
                "type": "string",
                "description": "Purchase of assets from noncharitable exempt organization?",
                "mode": "nullable",
            },
            {
                "name": "rentlsfacltscd",
                "type": "string",
                "description": "Rental of facilities or other assets?",
                "mode": "nullable",
            },
            {
                "name": "reimbrsmntscd",
                "type": "string",
                "description": "Reimbursements arrangements?",
                "mode": "nullable",
            },
            {
                "name": "loansguarcd",
                "type": "string",
                "description": "Loans or other guarantees?",
                "mode": "nullable",
            },
            {
                "name": "perfservicescd",
                "type": "string",
                "description": "Performance of services or membership or fundraising solicitations?",
                "mode": "nullable",
            },
            {
                "name": "sharngasstscd",
                "type": "string",
                "description": "Sharing of facilities equipment mailing lists other assets or paid employees?",
                "mode": "nullable",
            },
        ],
    )

    irs_990_pf_2015_transform_csv >> load_irs_990_pf_2015_to_bq
